/*
 *  tatengine_export.jsx
 *  TatEngine
 *  based on work of Tomek Cejner, Damien van Holten, Jens Bache-wiig
 *
 *  Created by Dmitrii Ivanov on 08/15/12.
 *  Copyright 2012 Tatem Games. All rights reserved.
 *
 */

// version 0.01a, from 08.15.12
// contacts : jimon@tatemgames.com

#target photoshop

function main()
{
	if(!documents.length)
		return;

	var doc = activeDocument;
	var oldPath = activeDocument.path;

	var outFolder = new Folder(oldPath + "/exported");
	if(!outFolder.exists)
		outFolder.create();

	var jsonFile = new File(oldPath + "/exported/export.json");
	jsonFile.encoding = "UTF8";
	jsonFile.open("w", "TEXT", "");
	jsonFile.write("{\n");
	jsonFile.write("\t\"name\" : \"root\",\n");
	jsonFile.write("\t\"type\" : \"group\",\n");
	jsonFile.write("\t\"width\" : " + Math.floor(doc.width.as("px")) + ",\n");
	jsonFile.write("\t\"height\" : " + Math.floor(doc.height.as("px")) + ",\n");

	jsonFile.write("\t\"childs\" :\n");
	jsonFile.write("\t[\n");

	var totalIndex = scanLayerSets(doc, 0, 0);

	jsonFile.write("\t],\n");
	jsonFile.write("\t\"totalIndexes\" : " + totalIndex + "\n");
	jsonFile.write("}\n");
	jsonFile.close();

	function tabs(lvl)
	{
		var str = "";
		for(var i = 0; i < lvl + 2; ++i)
			str += "\t";
		return str;
	}

	function escapeText(txt)
	{
		txt = txt.replace("\\", "\\\\");
		txt = txt.replace("\"", "\\\"");
		txt = txt.replace("\b", "\\b");
		txt = txt.replace("\f", "\\f");
		txt = txt.replace("\n", "\\n");
		txt = txt.replace("\r", "\\r");
		txt = txt.replace("\t", "\\t");
		return txt;
	}

	function scanLayerSets(el, index, lvl)
	{
		for(var i = 0; i < el.layers.length; i++)
		{
			var layer = el.layers[i];
			var layerName = escapeText(layer.name);
			var opacity = layer.opacity / 100.0 * 255.0;

			jsonFile.write(tabs(lvl) + "{\n");
			jsonFile.write(tabs(lvl + 1) + "\"name\" : \"" + layerName + "\",\n");
			jsonFile.write(tabs(lvl + 1) + "\"visible\" : " + layer.visible + ",\n");
			jsonFile.write(tabs(lvl + 1) + "\"opacity\" : " + layer.opacity + ",\n");

			if(layer.blendMode == BlendMode.COLORBLEND) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"colorBlend\",\n");
			else if(layer.blendMode == BlendMode.COLORBURN) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"colorBurn\",\n");
			else if(layer.blendMode == BlendMode.COLORDODGE) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"colorDodge\",\n");
			else if(layer.blendMode == BlendMode.DARKEN) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"darken\",\n");
			else if(layer.blendMode == BlendMode.DIFFERENCE) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"difference\",\n");
			else if(layer.blendMode == BlendMode.DISSOLVE) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"dissolve\",\n");
			else if(layer.blendMode == BlendMode.DIVIDE) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"divide\",\n");
			else if(layer.blendMode == BlendMode.EXCLUSION) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"exclusion\",\n");
			else if(layer.blendMode == BlendMode.HARDLIGHT) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"hardLight\",\n");
			else if(layer.blendMode == BlendMode.HARDMIX) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"hardMix\",\n");
			else if(layer.blendMode == BlendMode.HUE) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"hue\",\n");
			else if(layer.blendMode == BlendMode.LIGHTEN) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"lighten\",\n");
			else if(layer.blendMode == BlendMode.LINEARBURN) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"linearBurn\",\n");
			else if(layer.blendMode == BlendMode.LINEARDODGE) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"linearDodge\",\n");
			else if(layer.blendMode == BlendMode.LINEARLIGHT) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"linearLight\",\n");
			else if(layer.blendMode == BlendMode.LUMINOSITY) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"luminosity\",\n");
			else if(layer.blendMode == BlendMode.MULTIPLY) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"multiply\",\n");
			else if(layer.blendMode == BlendMode.NORMAL) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"normal\",\n");
			else if(layer.blendMode == BlendMode.OVERLAY) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"overlay\",\n");
			else if(layer.blendMode == BlendMode.PASSTHROUGH) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"passthrough\",\n");
			else if(layer.blendMode == BlendMode.PINLIGHT) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"pinLight\",\n");
			else if(layer.blendMode == BlendMode.SATURATION) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"saturation\",\n");
			else if(layer.blendMode == BlendMode.SCREEN) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"screen\",\n");
			else if(layer.blendMode == BlendMode.SOFTLIGHT) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"softLight\",\n");
			else if(layer.blendMode == BlendMode.SUBTRACT) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"subtract\",\n");
			else if(layer.blendMode == BlendMode.VIVIDLIGHT) jsonFile.write(tabs(lvl + 1) + "\"blendMode\" : \"vividLight\",\n");

			if(layer.typename == "LayerSet")
			{
				jsonFile.write(tabs(lvl + 1) + "\"type\" : \"group\",\n");
				jsonFile.write(tabs(lvl + 1) + "\"childs\" :\n");
				jsonFile.write(tabs(lvl + 1) + "[\n");

				index = scanLayerSets(layer, index, lvl + 2);

				jsonFile.write(tabs(lvl + 1) + "]\n");
			}
			else
			{
				jsonFile.write(tabs(lvl + 1) + "\"nodeIndex\" : " + index + ",\n");

				index++;

				var xoffset = layer.bounds[0].as("px");
				var yoffset = layer.bounds[1].as("px");
				
				if(layer.kind == LayerKind.TEXT)
				{
					var textItem = layer.textItem;

					var textContents = escapeText(textItem.contents);

					jsonFile.write(tabs(lvl + 1) + "\"type\" : \"text\",\n");
					jsonFile.write(tabs(lvl + 1) + "\"text\" : \"" + textContents + "\",\n");

					// hack from qml exporter
					yoffset -= textItem.size.as("px") / 4;

					jsonFile.write(tabs(lvl + 1) + "\"fontSize\" : " + Math.floor(textItem.size.as("px")) + ",\n");
					jsonFile.write(tabs(lvl + 1) + "\"fontFamily\" : \"" + textItem.font + "\",\n");

					if(textItem.font.indexOf("Bold") != -1)
						jsonFile.write(tabs(lvl + 1) + "\"fontBold\" : \"true\",\n");
					else
						jsonFile.write(tabs(lvl + 1) + "\"fontBold\" : \"false\",\n");

					if(textItem.font.indexOf("Italic") != -1)
						jsonFile.write(tabs(lvl + 1) + "\"fontItalic\" : \"true\",\n");
					else
						jsonFile.write(tabs(lvl + 1) + "\"fontItalic\" : \"false\",\n");

					jsonFile.write(tabs(lvl + 1) + "\"colorRed\" : " + Math.floor(textItem.color.rgb.red) + ",\n");
					jsonFile.write(tabs(lvl + 1) + "\"colorGreen\" : " + Math.floor(textItem.color.rgb.green) + ",\n");
					jsonFile.write(tabs(lvl + 1) + "\"colorBlue\" : " + Math.floor(textItem.color.rgb.blue) + ",\n");

					//textItem.position - origin of text
					if((textItem.justification == Justification.CENTER) || (textItem.justification == Justification.CENTERJUSTIFIED) || (textItem.justification == Justification.FULLYJUSTIFIED))
						jsonFile.write(tabs(lvl + 1) + "\"halign\" : \"center\",\n");
					else if((textItem.justification == Justification.LEFT) || (textItem.justification == Justification.LEFTJUSTIFIED))
						jsonFile.write(tabs(lvl + 1) + "\"halign\" : \"left\",\n");
					else if((textItem.justification == Justification.RIGHT) || (textItem.justification == Justification.RIGHTJUSTIFIED))
						jsonFile.write(tabs(lvl + 1) + "\"halign\" : \"right\",\n");

					if(!textItem.useAutoLeading)
						jsonFile.write(tabs(lvl + 1) + "\"leading\" : " + textItem.leading.as("px") + ",\n");

					jsonFile.write(tabs(lvl + 1) + "\"tracking\" : " + textItem.tracking + ",\n");
				}
				else
				{
					var fileName = layerName.replace(/[ :\/\\*\?\"\<\>\|#]/g, "_") + ".png";

					if(fileName.length > 150)
						fileName = fileName.substring(0, 150);

					saveLayer(layer, fileName, oldPath, false);

					jsonFile.write(tabs(lvl + 1) + "\"type\" : \"image\",\n");
					jsonFile.write(tabs(lvl + 1) + "\"fileName\" : \"" + fileName + "\",\n");
				}

				jsonFile.write(tabs(lvl + 1) + "\"x\" : " + Math.floor(xoffset) + ",\n");
				jsonFile.write(tabs(lvl + 1) + "\"y\" : " + Math.floor(yoffset) + ",\n");
				jsonFile.write(tabs(lvl + 1) + "\"width\" : " + Math.floor(layer.bounds[2].as("px")) + ",\n");
				jsonFile.write(tabs(lvl + 1) + "\"height\" : " + Math.floor(layer.bounds[3].as("px")) + "\n");
			}

			if(i == el.layers.length - 1)
				jsonFile.write(tabs(lvl) + "}\n");
			else
				jsonFile.write(tabs(lvl) + "},\n");
		}

		return index;
	}

	function saveLayer(layer, lname, path, shouldMerge)
	{
		var opacity = layer.opacity;
		layer.opacity = 100.0;

		activeDocument.activeLayer = layer;
		dupLayers();
		if(shouldMerge === undefined || shouldMerge === true)
		{
			activeDocument.mergeVisibleLayers();
		}
		activeDocument.trim(TrimType.TRANSPARENT, true, true, true, true);
		var saveFile = File(path + "/exported/" + lname);
		SavePNG(saveFile);
		app.activeDocument.close(SaveOptions.DONOTSAVECHANGES);

		layer.opacity = opacity;
	}

	function dupLayers()
	{
		var desc143 = new ActionDescriptor();
		var ref73 = new ActionReference();
		ref73.putClass(charIDToTypeID('Dcmn'));
		desc143.putReference(charIDToTypeID('null'), ref73);
		desc143.putString(charIDToTypeID('Nm  '), activeDocument.activeLayer.name);
		var ref74 = new ActionReference();
		ref74.putEnumerated(charIDToTypeID('Lyr '), charIDToTypeID('Ordn'), charIDToTypeID('Trgt'));
		desc143.putReference(charIDToTypeID('Usng'), ref74);
		executeAction(charIDToTypeID('Mk  '), desc143, DialogModes.NO);
	};

	function SavePNG(saveFile)
	{
		var pngOpts = new ExportOptionsSaveForWeb; 
		pngOpts.format = SaveDocumentType.PNG
		pngOpts.PNG8 = false; 
		pngOpts.transparency = true; 
		pngOpts.interlaced = false; 
		pngOpts.quality = 100;
		activeDocument.exportDocument(new File(saveFile),ExportType.SAVEFORWEB,pngOpts); 
	}
};

main();
